@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/child.c,v 1.6 2005/09/16 14:44:11 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -30,13 +32,13 @@ static void
 force_fd(int oldfd, int newfd)
 {
 if (oldfd == newfd) return;
-close(newfd);
-dup2(oldfd, newfd);
-close(oldfd);
+(void)close(newfd);
+(void)dup2(oldfd, newfd);
+(void)close(oldfd);
 }
 
 
-
+#ifndef STAND_ALONE
 /*************************************************
 *   Build argv list and optionally re-exec Exim  *
 *************************************************/
@@ -199,7 +201,7 @@ Exim. Failure is signalled with EX_EXECFAILED, but this shouldn't occur! */
 if (pid == 0)
   {
   force_fd(pfd[pipe_read], 0);
-  close(pfd[pipe_write]);
+  (void)close(pfd[pipe_write]);
   if (debug_fd > 0) force_fd(debug_fd, 2);
   if (bounce_sender_authentication != NULL)
     child_exec_exim(CEE_EXEC_EXIT, FALSE, NULL, FALSE, 8,
@@ -215,7 +217,7 @@ if (pid == 0)
 pipe. */
 
 save_errno = errno;
-close(pfd[pipe_read]);
+(void)close(pfd[pipe_read]);
 
 /* Fork succeeded */
 
@@ -227,11 +229,11 @@ if (pid > 0)
 
 /* Fork failed */
 
-close(pfd[pipe_write]);
+(void)close(pfd[pipe_write]);
 errno = save_errno;
 return (pid_t)(-1);
 }
-
+#endif
 
 
 
@@ -261,7 +263,7 @@ Arguments:
                 process is placed
   wd          if not NULL, a path to be handed to chdir() in the new process
   make_leader if TRUE, make the new process a process group leader
-  
+
 Returns:      the pid of the created process or -1 if anything has gone wrong
 */
 
@@ -278,8 +280,8 @@ pid_t pid;
 if (pipe(inpfd) != 0) return (pid_t)(-1);
 if (pipe(outpfd) != 0)
   {
-  close(inpfd[pipe_read]);
-  close(inpfd[pipe_write]);
+  (void)close(inpfd[pipe_read]);
+  (void)close(inpfd[pipe_write]);
   return (pid_t)(-1);
   }
 
@@ -299,14 +301,14 @@ if (pid == 0)
   {
   if (make_leader && setpgid(0,0) < 0) goto CHILD_FAILED;
 
-  close(inpfd[pipe_write]);
+  (void)close(inpfd[pipe_write]);
   force_fd(inpfd[pipe_read], 0);
 
-  close(outpfd[pipe_read]);
+  (void)close(outpfd[pipe_read]);
   force_fd(outpfd[pipe_write], 1);
 
-  close(2);
-  dup2(1, 2);
+  (void)close(2);
+  (void)dup2(1, 2);
 
   /* Set the required environment. */
 
@@ -336,8 +338,8 @@ if (pid == 0)
 stdin pipe, and the writing end of the stdout pipe. */
 
 save_errno = errno;
-close(inpfd[pipe_read]);
-close(outpfd[pipe_write]);
+(void)close(inpfd[pipe_read]);
+(void)close(outpfd[pipe_write]);
 
 /* Fork succeeded; return the input/output pipes and the pid */
 
@@ -350,8 +352,8 @@ if (pid > 0)
 
 /* Fork failed; reset fork errno before returning */
 
-close(inpfd[pipe_write]);
-close(outpfd[pipe_read]);
+(void)close(inpfd[pipe_write]);
+(void)close(outpfd[pipe_read]);
 errno = save_errno;
 return (pid_t)(-1);
 }
