@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/dbfn.c,v 1.6 2005/06/27 14:29:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -59,66 +61,6 @@ log_write(0, LOG_MAIN, "Berkeley DB error: %s", msg);
 
 
 
-static enum {
-  PRIV_DROPPING, PRIV_DROPPED,
-  PRIV_RESTORING, PRIV_RESTORED
-} priv_state = PRIV_RESTORED;
-
-static uid_t priv_euid;
-static gid_t priv_egid;
-static gid_t priv_groups[EXIM_GROUPLIST_SIZE + 1];
-static int priv_ngroups;
-
-/* Inspired by OpenSSH's temporarily_use_uid(). Thanks! */
-
-static void
-priv_drop_temp(const uid_t temp_uid, const gid_t temp_gid)
-{
-if (priv_state != PRIV_RESTORED) _exit(EXIT_FAILURE);
-priv_state = PRIV_DROPPING;
-
-priv_euid = geteuid();
-if (priv_euid == root_uid)
-  {
-  priv_egid = getegid();
-  priv_ngroups = getgroups(nelem(priv_groups), priv_groups);
-  if (priv_ngroups < 0) _exit(EXIT_FAILURE);
-
-  if (priv_ngroups > 0 && setgroups(1, &temp_gid) != 0) _exit(EXIT_FAILURE);
-  if (setegid(temp_gid) != 0) _exit(EXIT_FAILURE);
-  if (seteuid(temp_uid) != 0) _exit(EXIT_FAILURE);
-
-  if (geteuid() != temp_uid) _exit(EXIT_FAILURE);
-  if (getegid() != temp_gid) _exit(EXIT_FAILURE);
-  }
-
-priv_state = PRIV_DROPPED;
-}
-
-/* Inspired by OpenSSH's restore_uid(). Thanks! */
-
-static void
-priv_restore(void)
-{
-if (priv_state != PRIV_DROPPED) _exit(EXIT_FAILURE);
-priv_state = PRIV_RESTORING;
-
-if (priv_euid == root_uid)
-  {
-  if (seteuid(priv_euid) != 0) _exit(EXIT_FAILURE);
-  if (setegid(priv_egid) != 0) _exit(EXIT_FAILURE);
-  if (priv_ngroups > 0 && setgroups(priv_ngroups, priv_groups) != 0) _exit(EXIT_FAILURE);
-
-  if (geteuid() != priv_euid) _exit(EXIT_FAILURE);
-  if (getegid() != priv_egid) _exit(EXIT_FAILURE);
-  }
-
-priv_state = PRIV_RESTORED;
-}
-
-
-
-
 /*************************************************
 *          Open and lock a database file         *
 *************************************************/
@@ -138,10 +80,6 @@ Returns:   NULL if the open failed, or the locking failed. After locking
 
            On success, dbblock is returned. This contains the dbm pointer and
            the fd of the locked lock file.
-
-There are some calls that use O_RDWR|O_CREAT for the flags. Having discovered
-this in December 2005, I'm not sure if this is correct or not, but for the
-moment I haven't changed them.
 */
 
 open_db *
@@ -149,10 +87,9 @@ dbfn_open(uschar *name, int flags, open_db *dbblock, BOOL lof)
 {
 int rc, save_errno;
 BOOL read_only = flags == O_RDONLY;
+BOOL created = FALSE;
 flock_t lock_data;
-uschar dirname[256], filename[256];
-
-DEBUG(D_hints_lookup) acl_level++;
+uschar buffer[256];
 
 /* The first thing to do is to open a separate file on which to lock. This
 ensures that Exim has exclusive use of the database before it even tries to
@@ -167,23 +104,20 @@ make the directory as well, just in case. We won't be doing this many times
 unnecessarily, because usually the lock file will be there. If the directory
 exists, there is no error. */
 
-snprintf(CS dirname, sizeof(dirname), "%s/db", spool_directory);
-snprintf(CS filename, sizeof(filename), "%s/%s.lockfile", dirname, name);
+sprintf(CS buffer, "%s/db/%s.lockfile", spool_directory, name);
 
-priv_drop_temp(exim_uid, exim_gid);
-if ((dbblock->lockfd = Uopen(filename, O_RDWR, EXIMDB_LOCKFILE_MODE)) < 0)
+if ((dbblock->lockfd = Uopen(buffer, O_RDWR, EXIMDB_LOCKFILE_MODE)) < 0)
   {
+  created = TRUE;
   (void)directory_make(spool_directory, US"db", EXIMDB_DIRECTORY_MODE, TRUE);
-  dbblock->lockfd = Uopen(filename, O_RDWR|O_CREAT, EXIMDB_LOCKFILE_MODE);
+  dbblock->lockfd = Uopen(buffer, O_RDWR|O_CREAT, EXIMDB_LOCKFILE_MODE);
   }
-priv_restore();
 
 if (dbblock->lockfd < 0)
   {
   log_write(0, LOG_MAIN, "%s",
-    string_open_failed(errno, "database lock file %s", filename));
+    string_open_failed(errno, "database lock file %s", buffer));
   errno = 0;      /* Indicates locking failure */
-  DEBUG(D_hints_lookup) acl_level--;
   return NULL;
   }
 
@@ -194,26 +128,25 @@ lock_data.l_type = read_only? F_RDLCK : F_WRLCK;
 lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
 
 DEBUG(D_hints_lookup|D_retry|D_route|D_deliver)
-  debug_printf_indent("locking %s\n", filename);
+  debug_printf("locking %s\n", buffer);
 
 sigalrm_seen = FALSE;
-ALARM(EXIMDB_LOCK_TIMEOUT);
+alarm(EXIMDB_LOCK_TIMEOUT);
 rc = fcntl(dbblock->lockfd, F_SETLKW, &lock_data);
-ALARM_CLR(0);
+alarm(0);
 
 if (sigalrm_seen) errno = ETIMEDOUT;
 if (rc < 0)
   {
-  log_write(0, LOG_MAIN|LOG_PANIC, "Failed to get %s lock for %s: %s",
-    read_only ? "read" : "write", filename,
-    errno == ETIMEDOUT ? "timed out" : strerror(errno));
+  log_write(0, LOG_MAIN, "Failed to get %s lock for %s: %s",
+    ((flags & O_RDONLY) != 0)? "read" : "write", buffer,
+    (errno == ETIMEDOUT)? "timed out" : strerror(errno));
   (void)close(dbblock->lockfd);
   errno = 0;       /* Indicates locking failure */
-  DEBUG(D_hints_lookup) acl_level--;
   return NULL;
   }
 
-DEBUG(D_hints_lookup) debug_printf_indent("locked  %s\n", filename);
+DEBUG(D_hints_lookup) debug_printf("locked %s\n", buffer);
 
 /* At this point we have an opened and locked separate lock file, that is,
 exclusive access to the database, so we can go ahead and open it. If we are
@@ -224,44 +157,85 @@ databases - often this is caused by non-matching db.h and the library. To make
 it easy to pin this down, there are now debug statements on either side of the
 open call. */
 
-snprintf(CS filename, sizeof(filename), "%s/%s", dirname, name);
+sprintf(CS buffer, "%s/db/%s", spool_directory, name);
+DEBUG(D_hints_lookup) debug_printf("EXIM_DBOPEN(%s)\n", buffer);
+EXIM_DBOPEN(buffer, flags, EXIMDB_MODE, &(dbblock->dbptr));
+DEBUG(D_hints_lookup) debug_printf("returned from EXIM_DBOPEN\n");
 
-priv_drop_temp(exim_uid, exim_gid);
-EXIM_DBOPEN(filename, dirname, flags, EXIMDB_MODE, &(dbblock->dbptr));
-if (!dbblock->dbptr && errno == ENOENT && flags == O_RDWR)
+if (dbblock->dbptr == NULL && errno == ENOENT && flags == O_RDWR)
   {
   DEBUG(D_hints_lookup)
-    debug_printf_indent("%s appears not to exist: trying to create\n", filename);
-  EXIM_DBOPEN(filename, dirname, flags|O_CREAT, EXIMDB_MODE, &(dbblock->dbptr));
+    debug_printf("%s appears not to exist: trying to create\n", buffer);
+  created = TRUE;
+  EXIM_DBOPEN(buffer, flags|O_CREAT, EXIMDB_MODE, &(dbblock->dbptr));
+  DEBUG(D_hints_lookup) debug_printf("returned from EXIM_DBOPEN\n");
   }
+
 save_errno = errno;
-priv_restore();
+
+/* If we are running as root and this is the first access to the database, its
+files will be owned by root. We want them to be owned by exim. We detect this
+situation by noting above when we had to create the lock file or the database
+itself. Because the different dbm libraries use different extensions for their
+files, I don't know of any easier way of arranging this than scanning the
+directory for files with the appropriate base name. At least this deals with
+the lock file at the same time. Also, the directory will typically have only
+half a dozen files, so the scan will be quick.
+
+This code is placed here, before the test for successful opening, because there
+was a case when a file was created, but the DBM library still returned NULL
+because of some problem. It also sorts out the lock file if that was created
+but creation of the database file failed. */
+
+if (created && geteuid() == root_uid)
+  {
+  DIR *dd;
+  struct dirent *ent;
+  uschar *lastname = Ustrrchr(buffer, '/') + 1;
+  int namelen = Ustrlen(name);
+
+  *lastname = 0;
+  dd = opendir(CS buffer);
+
+  while ((ent = readdir(dd)) != NULL)
+    {
+    if (Ustrncmp(ent->d_name, name, namelen) == 0)
+      {
+      struct stat statbuf;
+      Ustrcpy(lastname, ent->d_name);
+      if (Ustat(buffer, &statbuf) >= 0 && statbuf.st_uid != exim_uid)
+        {
+        DEBUG(D_hints_lookup) debug_printf("ensuring %s is owned by exim\n", buffer);
+        (void)Uchown(buffer, exim_uid, exim_gid);
+        }
+      }
+    }
+
+  closedir(dd);
+  }
 
 /* If the open has failed, return NULL, leaving errno set. If lof is TRUE,
-log the event - also for debugging - but debug only if the file just doesn't
-exist. */
+log the event - also for debugging - but not if the file just doesn't exist. */
 
-if (!dbblock->dbptr)
+if (dbblock->dbptr == NULL)
   {
-  if (lof && save_errno != ENOENT)
-    log_write(0, LOG_MAIN, "%s", string_open_failed(save_errno, "DB file %s",
-        filename));
-  else
-    DEBUG(D_hints_lookup)
-      debug_printf_indent("%s\n", CS string_open_failed(save_errno, "DB file %s",
-          filename));
+  if (save_errno != ENOENT)
+    {
+    if (lof)
+      log_write(0, LOG_MAIN, "%s", string_open_failed(save_errno, "DB file %s",
+        buffer));
+    else
+      DEBUG(D_hints_lookup)
+        debug_printf("%s", CS string_open_failed(save_errno, "DB file %s\n",
+          buffer));
+    }
   (void)close(dbblock->lockfd);
   errno = save_errno;
-  DEBUG(D_hints_lookup) acl_level--;
   return NULL;
   }
 
 DEBUG(D_hints_lookup)
-  debug_printf_indent("opened hints database %s: flags=%s\n", filename,
-    flags == O_RDONLY ? "O_RDONLY"
-    : flags == O_RDWR ? "O_RDWR"
-    : flags == (O_RDWR|O_CREAT) ? "O_RDWR|O_CREAT"
-    : "??");
+  debug_printf("opened hints database %s: flags=%x\n", buffer, flags);
 
 /* Pass back the block containing the opened database handle and the open fd
 for the lock. */
@@ -288,8 +262,6 @@ dbfn_close(open_db *dbblock)
 {
 EXIM_DBCLOSE(dbblock->dbptr);
 (void)close(dbblock->lockfd);
-DEBUG(D_hints_lookup)
-  { debug_printf_indent("closed hints database and lockfile\n"); acl_level--; }
 }
 
 
@@ -317,21 +289,17 @@ Returns: a pointer to the retrieved record, or
 */
 
 void *
-dbfn_read_with_length(open_db *dbblock, const uschar *key, int *length)
+dbfn_read_with_length(open_db *dbblock, uschar *key, int *length)
 {
 void *yield;
 EXIM_DATUM key_datum, result_datum;
-int klen = Ustrlen(key) + 1;
-uschar * key_copy = store_get(klen);
 
-memcpy(key_copy, key, klen);
-
-DEBUG(D_hints_lookup) debug_printf_indent("dbfn_read: key=%s\n", key);
+DEBUG(D_hints_lookup) debug_printf("dbfn_read: key=%s\n", key);
 
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(result_datum);      /* to be cleared before use. */
-EXIM_DATUM_DATA(key_datum) = CS key_copy;
-EXIM_DATUM_SIZE(key_datum) = klen;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 
 if (!EXIM_DBGET(dbblock->dbptr, key_datum, result_datum)) return NULL;
 
@@ -361,22 +329,18 @@ Returns:    the yield of the underlying dbm or db "write" function. If this
 */
 
 int
-dbfn_write(open_db *dbblock, const uschar *key, void *ptr, int length)
+dbfn_write(open_db *dbblock, uschar *key, void *ptr, int length)
 {
 EXIM_DATUM key_datum, value_datum;
 dbdata_generic *gptr = (dbdata_generic *)ptr;
-int klen = Ustrlen(key) + 1;
-uschar * key_copy = store_get(klen);
-
-memcpy(key_copy, key, klen);
 gptr->time_stamp = time(NULL);
 
-DEBUG(D_hints_lookup) debug_printf_indent("dbfn_write: key=%s\n", key);
+DEBUG(D_hints_lookup) debug_printf("dbfn_write: key=%s\n", key);
 
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(value_datum);       /* to be cleared before use. */
-EXIM_DATUM_DATA(key_datum) = CS key_copy;
-EXIM_DATUM_SIZE(key_datum) = klen;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 EXIM_DATUM_DATA(value_datum) = CS ptr;
 EXIM_DATUM_SIZE(value_datum) = length;
 return EXIM_DBPUT(dbblock->dbptr, key_datum, value_datum);
@@ -397,18 +361,12 @@ Returns: the yield of the underlying dbm or db "delete" function.
 */
 
 int
-dbfn_delete(open_db *dbblock, const uschar *key)
+dbfn_delete(open_db *dbblock, uschar *key)
 {
-int klen = Ustrlen(key) + 1;
-uschar * key_copy = store_get(klen);
-
-DEBUG(D_hints_lookup) debug_printf_indent("dbfn_delete: key=%s\n", key);
-
-memcpy(key_copy, key, klen);
 EXIM_DATUM key_datum;
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require clearing */
-EXIM_DATUM_DATA(key_datum) = CS key_copy;
-EXIM_DATUM_SIZE(key_datum) = klen;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 return EXIM_DBDEL(dbblock->dbptr, key_datum);
 }
 
@@ -437,8 +395,6 @@ EXIM_DATUM key_datum, value_datum;
 uschar *yield;
 value_datum = value_datum;    /* dummy; not all db libraries use this */
 
-DEBUG(D_hints_lookup) debug_printf_indent("dbfn_scan\n");
-
 /* Some dbm require an initialization */
 
 if (start) EXIM_DBCREATE_CURSOR(dbblock->dbptr, cursor);
@@ -559,7 +515,7 @@ while (Ufgets(buffer, 256, stdin) != NULL)
     odb = dbfn_open(s, O_RDWR, dbblock + i, TRUE);
     stop = clock();
 
-    if (odb)
+    if (odb != NULL)
       {
       current = i;
       printf("opened %d\n", current);
