@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/filter.c,v 1.7 2005/11/11 10:02:04 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -19,10 +21,9 @@ need to declare the structures first because some of them are recursive. */
 
 struct filter_cmd;
 struct condition_block;
-struct alias;
 
 union argtypes {
-  struct alias           *a;
+  struct string_item     *a;
   BOOL                    b;
   struct condition_block *c;
   struct filter_cmd      *f;
@@ -48,11 +49,6 @@ typedef struct condition_block {
   union argtypes right;
 } condition_block;
 
-typedef struct alias {
-  struct alias *next;
-  uschar *name;
-} alias;
-
 /* Miscellaneous other declarations */
 
 static uschar **error_pointer;
@@ -522,7 +518,7 @@ for (;;)
       c->type = cond_personal;
       for (;;)
         {
-        alias *aa;
+        string_item *aa;
         uschar *saveptr = ptr;
         ptr = nextword(ptr, buffer, sizeof(buffer), TRUE);
         if (*error_pointer != NULL) break;
@@ -533,8 +529,8 @@ for (;;)
           }
         ptr = nextitem(ptr, buffer, sizeof(buffer), TRUE);
         if (*error_pointer != NULL) break;
-        aa = store_get(sizeof(alias));
-        aa->name = string_copy(buffer);
+        aa = store_get(sizeof(string_item));
+        aa->text = string_copy(buffer);
         aa->next = c->left.a;
         c->left.a = aa;
         }
@@ -1402,60 +1398,6 @@ return TRUE;
 
 
 
-/*************************************************
-*   Test a header for the "personal" condition   *
-*************************************************/
-
-/* A bit more efficient than doing a lot of explicit expansions. We pass
-in a chain of alias blocks and a variable-length list of explicit text
-strings.
-
-Arguments:
-  s          header name, including the trailing colon
-  cond       value to return if the header contains any of the strings
-               or aliases
-  aliases    points to a chain of alias blocks
-  count      number of strings
-  ...        the strings
-
-Returns:     cond if the header exists and contains one of the aliases or
-               strings; otherwise !cond
-*/
-
-static BOOL
-test_header(uschar *s, BOOL cond, alias *aliases, int count, ...)
-{
-header_line *h;
-int slen = Ustrlen(s);
-
-for (h = header_list; h != NULL; h = h->next)
-  {
-  int i;
-  alias *a;
-  va_list ap;
-
-  if (h->type == htype_old || slen > h->slen ||
-    strncmpic(s, h->text, slen) != 0)
-      continue;
-
-  for (a = aliases; a != NULL; a = a->next)
-    if (strstric(h->text + slen, a->name, FALSE) != NULL) return cond;
-
-  va_start(ap, count);
-  for (i = 0; i < count; i++)
-    {
-    uschar *p = va_arg(ap, uschar *);
-    if (strstric(h->text + slen, p, FALSE) != NULL) return cond;
-    }
-  va_end(ap);
-  }
-
-return !cond;
-}
-
-
-
-
 /*************************************************
 *             Test a condition                   *
 *************************************************/
@@ -1496,59 +1438,12 @@ switch (c->type)
           test_condition(c->right.c, FALSE));
   break;
 
-  /* The personal test is meaningless in a system filter */
+  /* The personal test is meaningless in a system filter. The tests are now in
+  a separate function (so Sieve can use them). However, an Exim filter does not
+  scan Cc: (hence the FALSE argument). */
 
   case cond_personal:
-  if (system_filtering) yield = FALSE; else
-    {
-    alias *aliases = c->left.a;
-    uschar *self, *self_from, *self_to;
-    uschar *psself = NULL;
-    uschar *psself_from = NULL, *psself_to = NULL;
-    int to_count = 2;
-    int from_count = 5;
-
-    self = string_sprintf("%s@%s", deliver_localpart, deliver_domain);
-    self_from = rewrite_one(self, rewrite_from, NULL, FALSE, US"",
-      global_rewrite_rules);
-    self_to   = rewrite_one(self, rewrite_to, NULL, FALSE, US"",
-      global_rewrite_rules);
-    if (self_from == NULL) self_from = self;
-    if (self_to == NULL) self_to = self;
-
-    /* If there's a prefix or suffix set, we must include the prefixed/
-    suffixed version of the local part in the tests. */
-
-    if (deliver_localpart_prefix != NULL || deliver_localpart_suffix != NULL)
-      {
-      psself = string_sprintf("%s%s%s@%s",
-        (deliver_localpart_prefix == NULL)? US"" : deliver_localpart_prefix,
-        deliver_localpart,
-        (deliver_localpart_suffix == NULL)? US"" : deliver_localpart_suffix,
-        deliver_domain);
-      psself_from = rewrite_one(psself, rewrite_from, NULL, FALSE, US"",
-        global_rewrite_rules);
-      psself_to   = rewrite_one(psself, rewrite_to, NULL, FALSE, US"",
-        global_rewrite_rules);
-      if (psself_from == NULL) self_from = psself;
-      if (psself_to == NULL) self_to = psself;
-      to_count += 2;
-      from_count += 2;
-      }
-
-    /* Do all the necessary tests; the counts are adjusted for {pre,suf}fix */
-
-    yield = test_header(US"to:", TRUE, aliases, to_count, self, self_to,
-              psself, psself_to) &&
-            test_header(US"from:", FALSE, aliases, from_count, "server@",
-              "daemon@", "root@", self, self_from, psself, psself_from) &&
-            test_header(US"subject:", FALSE, NULL, 1, "circular") &&
-            test_header(US"auto-submitted:", FALSE, NULL, 1, "auto-") &&
-            test_header(US"precedence:", FALSE, NULL, 1, "bulk") &&
-            test_header(US"precedence:", FALSE, NULL, 1, "list") &&
-            test_header(US"precedence:", FALSE, NULL, 1, "junk") &&
-            (sender_address == NULL || sender_address[0] != 0);
-    }
+  yield = system_filtering? FALSE : filter_personal(c->left.a, FALSE);
   break;
 
   case cond_delivered:
@@ -1567,7 +1462,7 @@ switch (c->type)
   and filter testing and verification. */
 
   case cond_firsttime:
-  yield = filter_test != NULL || message_id[0] == 0 || deliver_firsttime;
+  yield = filter_test != FTEST_NONE || message_id[0] == 0 || deliver_firsttime;
   break;
 
   /* Only TRUE if a message is actually being processed; FALSE for address
@@ -1608,7 +1503,7 @@ switch (c->type)
 
     if (filter_thisaddress != NULL)
       {
-      if ((filter_test != NULL && debug_selector != 0) ||
+      if ((filter_test != FTEST_NONE && debug_selector != 0) ||
           (debug_selector & D_filter) != 0)
         {
         indent();
@@ -1683,7 +1578,7 @@ switch (c->type)
 
     case cond_matches:
     case cond_MATCHES:
-    if ((filter_test != NULL && debug_selector != 0) ||
+    if ((filter_test != FTEST_NONE && debug_selector != 0) ||
         (debug_selector & D_filter) != 0)
       {
       debug_printf("Match expanded arguments:\n");
@@ -1726,7 +1621,7 @@ switch (c->type)
   break;
   }
 
-if ((filter_test != NULL && debug_selector != 0) ||
+if ((filter_test != FTEST_NONE && debug_selector != 0) ||
     (debug_selector & D_filter) != 0)
   {
   indent();
@@ -1834,7 +1729,7 @@ while (commands != NULL)
       }
 
     filter_n[n[1]] += n[0];
-    if (filter_test != NULL) printf("Add %d to n%d\n", n[0], n[1]);
+    if (filter_test != FTEST_NONE) printf("Add %d to n%d\n", n[0], n[1]);
     break;
 
     /* A deliver command's argument must be a valid address. Its optional
@@ -1882,7 +1777,7 @@ while (commands != NULL)
 
     /* Test case: report what would happen */
 
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       printf("%seliver message to: %s%s%s%s\n",
@@ -1923,7 +1818,7 @@ while (commands != NULL)
 
     /* Test case: report what would happen */
 
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       if (mode < 0)
@@ -1960,7 +1855,7 @@ while (commands != NULL)
 
     case pipe_command:
     s = string_copy(commands->args[0].u);
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       printf("%sipe message to: %s%s\n", (commands->seen)?
@@ -2012,11 +1907,11 @@ while (commands != NULL)
     if (log_mode == -1) log_mode = 0600;
     if (log_fd >= 0)
       {
-      close(log_fd);
+      (void)close(log_fd);
       log_fd = -1;
       }
     log_filename = expargs[0];
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       printf("%sogfile %s\n", (commands->seen)? "Seen l" : "L", log_filename);
@@ -2026,7 +1921,7 @@ while (commands != NULL)
     case logwrite_command:
     s = expargs[0];
 
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       printf("%sogwrite \"%s\"\n", (commands->seen)? "Seen l" : "L",
@@ -2088,7 +1983,7 @@ while (commands != NULL)
       int subtype = commands->args[1].i;
       s = expargs[0];
 
-      if (filter_test != NULL)
+      if (filter_test != FTEST_NONE)
         printf("Headers %s \"%s\"\n", (subtype == TRUE)? "add" :
           (subtype == FALSE)? "remove" : "charset", string_printing(s));
 
@@ -2100,31 +1995,19 @@ while (commands != NULL)
           header_add(htype_other, "%s%s", s, (s[Ustrlen(s)-1] == '\n')?
             "" : "\n");
           header_last->type = header_checkname(header_last, FALSE);
+          if (header_last->type >= 'a') header_last->type = htype_other;
           }
         }
 
       else if (subtype == FALSE)
         {
-        header_line *h;
-        for (h = header_list; h != NULL; h = h->next)
-          {
-          int sep = 0;
-          uschar *ss, *tt;
-          uschar *list = s;
-          uschar buffer[128];
-          if (h->type == htype_old) continue;
-          while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
-                 != NULL)
-            {
-            int len = Ustrlen(ss);
-            if (strncmpic(h->text, ss, len) != 0) continue;
-            tt = h->text + len;
-            while (*tt == ' ' || *tt == '\t') tt++;
-            if (*tt == ':') break;
-            }
-          if (ss == NULL) continue;
-          h->type = htype_old;
-          }
+        int sep = 0;
+        uschar *ss;
+        uschar *list = s;
+        uschar buffer[128];
+        while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
+               != NULL)
+          header_remove(0, ss);
         }
 
       /* This setting lasts only while the filter is running; on exit, the
@@ -2159,7 +2042,7 @@ while (commands != NULL)
     fmsg = string_printing(fmsg);
     *error_pointer = fmsg;
 
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       printf("%c%s text \"%s\"\n", toupper(ff_name[0]), ff_name+1, fmsg);
@@ -2168,7 +2051,7 @@ while (commands != NULL)
     return ff_ret;
 
     case finish_command:
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       indent();
       printf("%sinish\n", (commands->seen)? "Seen f" : "F");
@@ -2208,7 +2091,7 @@ while (commands != NULL)
     case vacation_command:
     if (return_path == NULL || return_path[0] == 0)
       {
-      if (filter_test != NULL)
+      if (filter_test != FTEST_NONE)
         printf("%s command ignored because return_path is empty\n",
           command_list[commands->command]);
       else DEBUG(D_filter) debug_printf("%s command ignored because return_path "
@@ -2219,8 +2102,8 @@ while (commands != NULL)
     /* Check the contents of the strings. The type of string can be deduced
     from the value of i.
 
-    . If i is equal to mailarg_index_text its a text string for the body, where
-      anything goes.
+    . If i is equal to mailarg_index_text it's a text string for the body,
+      where anything goes.
 
     . If i is > mailarg_index_text, we are dealing with a file name, which
       cannot contain non-printing characters.
@@ -2297,7 +2180,7 @@ while (commands != NULL)
 
     /* Proceed with mail or vacation command */
 
-    if (filter_test != NULL)
+    if (filter_test != FTEST_NONE)
       {
       uschar *to = commands->args[mailarg_index_to].u;
       indent();
@@ -2324,12 +2207,16 @@ while (commands != NULL)
     else
       {
       uschar *tt;
+      uschar *log_addr = NULL;
       uschar *to = commands->args[mailarg_index_to].u;
+      int size = 0;
+      int ptr = 0;
+
       if (to == NULL) to = expand_string(US"$reply_address");
       while (isspace(*to)) to++;
 
-      for (tt = to; *tt != 0; tt++)     /* Get rid of newlines so that */
-        if (*tt == '\n') *tt = ' ';     /* the eventual log line is OK */
+      for (tt = to; *tt != 0; tt++)     /* Get rid of newlines */
+        if (*tt == '\n') *tt = ' ';
 
       DEBUG(D_filter)
         {
@@ -2352,9 +2239,54 @@ while (commands != NULL)
           }
         }
 
-      /* Create the "address" for the autoreply */
+      /* Create the "address" for the autoreply. This is used only for logging,
+      as the actual recipients are extracted from the To: line by -t. We use the
+      same logic here to extract the working addresses (there may be more than
+      one). Just in case there are a vast number of addresses, stop when the
+      string gets too long. */
 
-      addr = deliver_make_addr(string_sprintf(">%.256s", to), FALSE);
+      tt = to;
+      while (*tt != 0)
+        {
+        uschar *ss = parse_find_address_end(tt, FALSE);
+        uschar *recipient, *errmess;
+        int start, end, domain;
+        int temp = *ss;
+
+        *ss = 0;
+        recipient = parse_extract_address(tt, &errmess, &start, &end, &domain,
+          FALSE);
+        *ss = temp;
+
+        /* Ignore empty addresses and errors; an error will occur later if
+        there's something really bad. */
+
+        if (recipient != NULL)
+          {
+          log_addr = string_cat(log_addr, &size, &ptr,
+            (log_addr == NULL)? US">" : US",", 1);
+          log_addr = string_cat(log_addr, &size, &ptr, recipient,
+            Ustrlen(recipient));
+          }
+
+        /* Check size */
+
+        if (ptr > 256)
+          {
+          log_addr = string_cat(log_addr, &size, &ptr, US", ...", 5);
+          break;
+          }
+
+        /* Move on past this address */
+
+        tt = ss + (*ss? 1:0);
+        while (isspace(*tt)) tt++;
+        }
+
+      if (log_addr == NULL) log_addr = string_sprintf("invalid-to-line");
+        else log_addr[ptr] = 0;
+
+      addr = deliver_make_addr(log_addr, FALSE);
       setflag(addr, af_pfr);
       if (commands->noerror) setflag(addr, af_ignore_error);
       addr->next = *generated;
@@ -2367,6 +2299,7 @@ while (commands != NULL)
       addr->reply->expand_forbid = expand_forbid;
       addr->reply->return_message =
         commands->args[mailarg_index_return].u != NULL;
+      addr->reply->once_repeat = 0;
 
       if (commands->args[mailarg_index_once_repeat].u != NULL)
         {
@@ -2394,10 +2327,10 @@ while (commands != NULL)
     break;
 
     case testprint_command:
-    if (filter_test != NULL || (debug_selector & D_filter) != 0)
+    if (filter_test != FTEST_NONE || (debug_selector & D_filter) != 0)
       {
       uschar *s = string_printing(expargs[0]);
-      if (filter_test == NULL)
+      if (filter_test == FTEST_NONE)
         debug_printf("Filter: testprint: %s\n", s);
       else
         printf("Testprint: %s\n", s);
@@ -2412,6 +2345,132 @@ return filter_delivered? FF_DELIVERED : FF_NOTDELIVERED;
 
 
 
+/*************************************************
+*        Test for a personal message             *
+*************************************************/
+
+/* This function is global so that it can also be called from the code that
+implements Sieve filters.
+
+Arguments:
+  aliases    a chain of aliases
+  scan_cc    TRUE if Cc: and Bcc: are to be scanned (Exim filters do not)
+
+Returns:     TRUE if the message is deemed to be personal
+*/
+
+BOOL
+filter_personal(string_item *aliases, BOOL scan_cc)
+{
+uschar *self, *self_from, *self_to;
+uschar *psself = NULL, *psself_from = NULL, *psself_to = NULL;
+void *reset_point = store_get(0);
+BOOL yield;
+header_line *h;
+int to_count = 2;
+int from_count = 9;
+
+/* If any header line in the message is a defined "List-" header field, it is
+not a personal message. We used to check for any header line that started with
+"List-", but this was tightened up for release 4.54. The check is now for
+"List-Id", defined in RFC 2929, or "List-Help", "List-Subscribe", "List-
+Unsubscribe", "List-Post", "List-Owner" or "List-Archive", all of which are
+defined in RFC 2369. We also scan for "Auto-Submitted"; if it is found to
+contain any value other than "no", the message is not personal (RFC 3834).
+Previously the test was for "auto-". */
+
+for (h = header_list; h != NULL; h = h->next)
+  {
+  uschar *s;
+  if (h->type == htype_old) continue;
+
+  if (strncmpic(h->text, US"List-", 5) == 0)
+    {
+    s = h->text + 5;
+    if (strncmpic(s, US"Id:", 3) == 0 ||
+        strncmpic(s, US"Help:", 5) == 0 ||
+        strncmpic(s, US"Subscribe:", 10) == 0 ||
+        strncmpic(s, US"Unsubscribe:", 12) == 0 ||
+        strncmpic(s, US"Post:", 5) == 0 ||
+        strncmpic(s, US"Owner:", 6) == 0 ||
+        strncmpic(s, US"Archive:", 8) == 0)
+      return FALSE;
+    }
+
+  else if (strncmpic(h->text, US"Auto-submitted:", 15) == 0)
+    {
+    s = h->text + 15;
+    while (isspace(*s)) s++;
+    if (strncmpic(s, US"no", 2) != 0) return FALSE;
+    s += 2;
+    while (isspace(*s)) s++;
+    if (*s != 0) return FALSE;
+    }
+  }
+
+/* Set up "my" address */
+
+self = string_sprintf("%s@%s", deliver_localpart, deliver_domain);
+self_from = rewrite_one(self, rewrite_from, NULL, FALSE, US"",
+  global_rewrite_rules);
+self_to   = rewrite_one(self, rewrite_to, NULL, FALSE, US"",
+  global_rewrite_rules);
+
+
+if (self_from == NULL) self_from = self;
+if (self_to == NULL) self_to = self;
+
+/* If there's a prefix or suffix set, we must include the prefixed/
+suffixed version of the local part in the tests. */
+
+if (deliver_localpart_prefix != NULL || deliver_localpart_suffix != NULL)
+  {
+  psself = string_sprintf("%s%s%s@%s",
+    (deliver_localpart_prefix == NULL)? US"" : deliver_localpart_prefix,
+    deliver_localpart,
+    (deliver_localpart_suffix == NULL)? US"" : deliver_localpart_suffix,
+    deliver_domain);
+  psself_from = rewrite_one(psself, rewrite_from, NULL, FALSE, US"",
+    global_rewrite_rules);
+  psself_to   = rewrite_one(psself, rewrite_to, NULL, FALSE, US"",
+    global_rewrite_rules);
+  if (psself_from == NULL) psself_from = psself;
+  if (psself_to == NULL) psself_to = psself;
+  to_count += 2;
+  from_count += 2;
+  }
+
+/* Do all the necessary tests; the counts are adjusted for {pre,suf}fix */
+
+yield =
+  (
+  header_match(US"to:", TRUE, TRUE, aliases, to_count, self, self_to, psself,
+               psself_to) ||
+    (scan_cc &&
+       (
+       header_match(US"cc:", TRUE, TRUE, aliases, to_count, self, self_to,
+                             psself, psself_to)
+       ||
+       header_match(US"bcc:", TRUE, TRUE, aliases, to_count, self, self_to,
+                              psself, psself_to)
+       )
+    )
+  ) &&
+
+  header_match(US"from:", TRUE, FALSE, aliases, from_count, "^server@",
+    "^daemon@", "^root@", "^listserv@", "^majordomo@", "^.*?-request@",
+    "^owner-[^@]+@", self, self_from, psself, psself_from) &&
+
+  header_match(US"precedence:", FALSE, FALSE, NULL, 3, "bulk","list","junk") &&
+
+  (sender_address == NULL || sender_address[0] != 0);
+
+store_reset(reset_point);
+return yield;
+}
+
+
+
 /*************************************************
 *            Interpret a mail filter file        *
 *************************************************/
@@ -2477,7 +2536,7 @@ ptr = nextsigchar(ptr, TRUE);
 if (read_command_list(&ptr, &lastcmdptr, FALSE))
   yield = interpret_commands(commands, generated);
 
-if (filter_test != NULL || (debug_selector & D_filter) != 0)
+if (filter_test != FTEST_NONE || (debug_selector & D_filter) != 0)
   {
   uschar *s = US"";
   switch(yield)
@@ -2510,14 +2569,14 @@ if (filter_test != NULL || (debug_selector & D_filter) != 0)
     break;
     }
 
-  if (filter_test != NULL) printf("%s\n", CS s);
+  if (filter_test != FTEST_NONE) printf("%s\n", CS s);
     else debug_printf("%s\n", s);
   }
 
 /* Close the log file if it was opened, and kill off any numerical variables
 before returning. Reset the header decoding charset. */
 
-if (log_fd >= 0) close(log_fd);
+if (log_fd >= 0) (void)close(log_fd);
 expand_nmax = -1;
 filter_running = FALSE;
 headers_charset = save_headers_charset;
