@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/ip.c,v 1.3 2005/06/27 14:29:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for doing things with sockets. With the advent of IPv6 this has
@@ -247,7 +249,7 @@ if (rc >= 0) return 0;
 /* A failure whose error code is "Interrupted system call" is in fact
 an externally applied timeout if the signal handler has been run. */
 
-close(sock);
+(void)close(sock);
 errno = (save_errno == EINTR && sigalrm_seen)? ETIMEDOUT : save_errno;
 return -1;
 }
@@ -303,6 +305,7 @@ ip_recv(int sock, uschar *buffer, int buffsize, int timeout)
 {
 fd_set select_inset;
 struct timeval tv;
+int start_recv = time(NULL);
 int rc;
 
 /* Wait until the socket is ready */
@@ -313,19 +316,28 @@ for (;;)
   FD_SET (sock, &select_inset);
   tv.tv_sec = timeout;
   tv.tv_usec = 0;
+
+  DEBUG(D_transport) debug_printf("waiting for data on socket\n");
   rc = select(sock + 1, (SELECT_ARG2_TYPE *)&select_inset, NULL, NULL, &tv);
 
   /* If some interrupt arrived, just retry. We presume this to be rare,
   but it can happen (e.g. the SIGUSR1 signal sent by exiwhat causes
-  select() to exit). */
+  select() to exit).
+
+  Aug 2004: Somebody set up a cron job that ran exiwhat every 2 minutes, making
+  the interrupt not at all rare. Since the timeout is typically more than 2
+  minutes, the effect was to block the timeout completely. To prevent this
+  happening again, we do an explicit time test. */
 
   if (rc < 0 && errno == EINTR)
     {
-    HDEBUG(D_any) debug_printf("EINTR while selecting for socket read\n");
-    continue;
+    DEBUG(D_transport) debug_printf("EINTR while waiting for socket data\n");
+    if (time(NULL) - start_recv < timeout) continue;
+    DEBUG(D_transport) debug_printf("total wait time exceeds timeout\n");
     }
 
-  /* Handle a timeout, and treat any other select error as a timeout. */
+  /* Handle a timeout, and treat any other select error as a timeout, including
+  an EINTR when we have been in this loop for longer than timeout. */
 
   if (rc <= 0)
     {
