@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/lookups/lsearch.c,v 1.11 2009/11/16 19:50:38 nm4 Exp $ */
+/* $Cambridge: exim/exim-src/src/lookups/lsearch.c,v 1.5 2005/11/15 09:44:33 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -107,7 +107,6 @@ for (last_was_eol = TRUE;
   int ptr, size;
   int p = Ustrlen(buffer);
   int linekeylength;
-  BOOL this_is_comment;
   uschar *yield;
   uschar *s = buffer;
 
@@ -256,7 +255,6 @@ for (last_was_eol = TRUE;
 
   Initialize, and copy the first segment of data. */
 
-  this_is_comment = FALSE;
   size = 100;
   ptr = 0;
   yield = store_get(size);
@@ -287,13 +285,11 @@ for (last_was_eol = TRUE;
 
     if (last_was_eol)
       {
-      this_is_comment = (this_is_comment || (buffer[0] == 0 || buffer[0] == '#'));
-      if (this_is_comment) continue;
+      if (buffer[0] == 0 || buffer[0] == '#') continue;
       if (!isspace((uschar)buffer[0])) break;
       while (isspace((uschar)*s)) s++;
       *(--s) = ' ';
       }
-    if (this_is_comment) continue;
 
     /* Join a physical or logical line continuation onto the result string. */
 
@@ -382,7 +378,7 @@ iplsearch_find(void *handle, uschar *filename, uschar *keystring, int length,
 {
 do_cache = do_cache;  /* Keep picky compilers happy */
 if ((length == 1 && keystring[0] == '*') ||
-    string_is_ip_address(keystring, NULL) != 0)
+    string_is_ip_address(keystring, NULL) > 0)
   {
   return internal_lsearch_find(handle, filename, keystring, length, result,
     errmsg, LSEARCH_IP);
