@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/routers/dnslookup.c,v 1.12 2009/11/16 19:50:38 nm4 Exp $ */
+/* $Cambridge: exim/exim-src/src/routers/dnslookup.c,v 1.8 2005/09/19 18:00:47 fanf2 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -323,7 +323,7 @@ for (;;)
     else
       {
       addr->message = US"all relevant MX records point to non-existent hosts";
-      if (!allow_mx_to_ip && string_is_ip_address(h.name, NULL) != 0)
+      if (!allow_mx_to_ip && string_is_ip_address(h.name, NULL) > 0)
         {
         addr->user_message =
           string_sprintf("It appears that the DNS operator for %s\n"
