@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/routers/manualroute.h,v 1.3 2005/09/12 15:09:55 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the manualroute router */
@@ -29,7 +31,7 @@ extern manualroute_router_options_block manualroute_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int manualroute_router_entry(router_instance *, address_item *,
-  struct passwd *, BOOL, address_item **, address_item **,
+  struct passwd *, int, address_item **, address_item **,
   address_item **, address_item **);
 
 extern void manualroute_router_init(router_instance *);
