@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/routers/redirect.h,v 1.6 2005/09/12 15:09:55 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the redirect router */
@@ -23,6 +25,9 @@ typedef struct {
   uschar *include_directory;
   uschar *pipe_transport_name;
   uschar *reply_transport_name;
+  uschar *sieve_subaddress;
+  uschar *sieve_useraddress;
+  uschar *sieve_vacation_directory;
   uschar *syntax_errors_text;
   uschar *syntax_errors_to;
   uschar *qualify_domain;
@@ -30,6 +35,14 @@ typedef struct {
   uid_t  *owners;
   gid_t  *owngroups;
 
+#ifdef EXPERIMENTAL_SRS
+  uschar *srs;
+  uschar *srs_alias;
+  uschar *srs_condition;
+  uschar *srs_dbinsert;
+  uschar *srs_dbselect;
+#endif
+
   int   modemask;
   int   bit_options;
   BOOL  check_ancestor;
@@ -56,7 +69,7 @@ extern redirect_router_options_block redirect_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int redirect_router_entry(router_instance *, address_item *,
-  struct passwd *, BOOL, address_item **, address_item **,
+  struct passwd *, int, address_item **, address_item **,
   address_item **, address_item **);
 
 extern void redirect_router_init(router_instance *);
